#!/usr/bin/env python
# Copyright 2011 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
        name='django_factory',
        version='0.10',
        description=('Generic factory for creating instances of Django '
                     'models in tests.'),
        author='James Westby',
        author_email='james.westby@canonical.com',
        url='https://launchpad.net/django_factory',
        packages=['django_factory'],
        install_requires=['django>=1.1', 'testtools'],
        license="Apache-2",
        long_description=read('README'),
        classifiers=[
            'Development Status :: 4 - Beta',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 2.6',
            'Topic :: Database',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Software Development :: Testing',
        ],
)
